@@ -8,6 +8,7 @@ raven.contrib.django.utils
 
 from __future__ import absolute_import
 
+import os
 from django.conf import settings
 
 
@@ -20,21 +21,34 @@ def linebreak_iter(template_source):
     yield len(template_source) + 1
 
 
-def get_data_from_template(source):
-    origin, (start, end) = source
-    template_source = origin.reload()
+def get_data_from_template(source, debug=None):
+    if debug is not None:
+        start = debug['start']
+        end = debug['end']
+        source_lines = debug['source_lines']
+        lineno = debug['line']
+        filename = debug['name']
+        culprit = filename.split('/templates/')[-1]
+    elif source:
+        origin, (start, end) = source
+        filename = culprit = getattr(origin, 'loadname', None)
+        template_source = origin.reload()
+        lineno = None
+        upto = 0
+        source_lines = []
+        for num, next in enumerate(linebreak_iter(template_source)):
+            if start >= upto and end <= next:
+                lineno = num
+            source_lines.append(template_source[upto:next])
+            upto = next
 
-    lineno = None
-    upto = 0
-    source_lines = []
-    for num, next in enumerate(linebreak_iter(template_source)):
-        if start >= upto and end <= next:
-            lineno = num
-        source_lines.append(template_source[upto:next])
-        upto = next
+        if not source_lines or lineno is None:
+            return {}
+    else:
+        raise TypeError('Source or debug needed')
 
-    if not source_lines or lineno is None:
-        return {}
+    if filename is None:
+        filename = '<unknown filename>'
 
     pre_context = source_lines[max(lineno - 3, 0):lineno]
     post_context = source_lines[(lineno + 1):(lineno + 4)]
@@ -42,14 +56,14 @@ def get_data_from_template(source):
 
     return {
         'template': {
-            'filename': origin.loadname,
-            'abs_path': origin.name,
+            'filename': os.path.basename(filename),
+            'abs_path': filename,
             'pre_context': pre_context,
             'context_line': context_line,
             'lineno': lineno,
             'post_context': post_context,
         },
-        'culprit': origin.loadname,
+        'culprit': culprit,
     }
 
 
